//
// File:        Inverter.java
// Package:     gov.llnl.babel.symbols
// Copyright:   (c) 2006 The Regents of the University of California
// Release:     $Name$
// Revision:    @(#) $Revision: 5298 $
// Date:        $Date: 2006-03-27 08:19:36 -0800 (Mon, 27 Mar 2006) $
// Description: Visitor to invert an expression
// 

package gov.llnl.babel.symbols;

import gov.llnl.babel.symbols.AssertionExpression;
import gov.llnl.babel.symbols.AssertionException;
import gov.llnl.babel.symbols.BinaryExpression;
import gov.llnl.babel.symbols.ExprVisitor;
import gov.llnl.babel.symbols.IdentifierLiteral;
import gov.llnl.babel.symbols.UnaryExpression;

public class Inverter extends ExprVisitor {

  private static class HasVariable extends ExprVisitor {
    public Object visitIdentifier(IdentifierLiteral il, Object data) {
      return il;
    }
  }
  
  public Object visitBinaryExpression(BinaryExpression be, Object data)   {
    final boolean lhsHasVariable = null !=
      be.getLeftExpression().accept(new HasVariable(), null);
    final boolean rhsHasVariable = null !=
      be.getRightExpression().accept(new HasVariable(), null);
    try {
      if (lhsHasVariable) {
        switch(be.getOp()) {
        case BinaryExpression.DIVIDE:
          return be.getLeftExpression().
            accept(this, new BinaryExpression((AssertionExpression)data,
                                              BinaryExpression.MULTIPLY,
                                              be.getRightExpression()));
        case BinaryExpression.MULTIPLY:
          return be.getLeftExpression().
            accept(this, new BinaryExpression((AssertionExpression)data,
                                              BinaryExpression.DIVIDE,
                                              be.getRightExpression()));
        case BinaryExpression.PLUS:
          return be.getLeftExpression().
            accept(this, new BinaryExpression((AssertionExpression)data,
                                              BinaryExpression.MINUS,
                                              be.getRightExpression()));
        case BinaryExpression.MINUS:
          return be.getLeftExpression().
            accept(this, new BinaryExpression((AssertionExpression)data,
                                              BinaryExpression.PLUS,
                                              be.getRightExpression()));
        }
      } else if (rhsHasVariable) {
        switch(be.getOp()) {
        case BinaryExpression.DIVIDE:
          return be.getRightExpression().
            accept(this, new BinaryExpression((AssertionExpression)data,
                                              BinaryExpression.DIVIDE,
                                              be.getLeftExpression()));
        case BinaryExpression.MULTIPLY:
          return be.getRightExpression().
            accept(this, new BinaryExpression((AssertionExpression)data,
                                              BinaryExpression.DIVIDE,
                                              be.getLeftExpression()));
        case BinaryExpression.PLUS:
          return be.getRightExpression().
            accept(this, new BinaryExpression((AssertionExpression)data,
                                              BinaryExpression.MINUS,
                                              be.getLeftExpression()));
        case BinaryExpression.MINUS:
          return be.getRightExpression().
            accept(this, new BinaryExpression(be.getLeftExpression(),
                                              BinaryExpression.MINUS,
                                              (AssertionExpression)data));
        }
      }
    }
    catch (AssertionException aexcpt) {
    }
    return null;
  }

  public Object visitUnaryExpression(UnaryExpression ue, Object data) {
    try {
      if ( null != ue.getExpression().accept(new HasVariable(), null)) {
        switch(ue.getOp()) {
        case UnaryExpression.PLUS: 
          return ue.getExpression().accept(this, data);
        case UnaryExpression.MINUS:
          return ue.getExpression().
            accept(this, new UnaryExpression(UnaryExpression.MINUS,
                                             (AssertionExpression)data));
        }
      }
    }
    catch (AssertionException ae) {
    }
    return null;
  }

  public static AssertionExpression invertExpr(AssertionExpression ae,
                                               String lengthExpr) {
    try {
      return (AssertionExpression)
        ae.accept(new Inverter(), new IdentifierLiteral(lengthExpr));
    }
    catch (AssertionException aexcpt) {
    }
    return null;
  }
}
