!
! File:       exceptionclient.F90
! Copyright:  (c) 2001-2002 The Regents of the University of California
! Revision:   @(#) $Revision: 5555 $
! Date:       $Date: 2006-06-09 10:40:02 -0700 (Fri, 09 Jun 2006) $
! Description:Simple F90 exception test client
!
!
#include "sidl_BaseInterface_fAbbrev.h"
#include "sidl_SIDLException_fAbbrev.h"
#include "ExceptionTest_Fib_fAbbrev.h"
#include "synch_RegOut_fAbbrev.h"

subroutine starttest(number)
  use sidl
  use synch_RegOut
  use sidl_BaseInterface
  implicit none
  integer (kind=sidl_int) :: number
  type(sidl_BaseInterface_t) :: throwaway
  type(synch_RegOut_t) :: tracker
  call getInstance(tracker, throwaway)
  call startPart(tracker, number, throwaway)
  call deleteRef(tracker, throwaway)
end subroutine starttest

subroutine reporttest(test, number)
  use sidl
  use synch_RegOut
  use synch_ResultType
  use sidl_BaseInterface
  implicit none
  integer (kind=sidl_int) :: number
  type(synch_RegOut_t) :: tracker
  type(sidl_BaseInterface_t) :: throwaway
  logical                        :: test
  call getInstance(tracker, throwaway)
  if (test) then
     call endPart(tracker, number, PASS, throwaway)
  else
     call endPart(tracker, number, FAIL, throwaway)
  endif
  number = number + 1
  call deleteRef(tracker, throwaway)
end subroutine reporttest

subroutine reportexc(sExcept)
  use sidl_SIDLException
  use sidl_BaseInterface
  implicit none
  type(sidl_BaseInterface_t) :: throwaway
  type(sidl_SIDLException_t)  :: sExcept
  character (len=100)         :: msg
  character (len=1024)        :: trace

  call getNote(sExcept, msg, throwaway)
  write (6, *) msg
  call getTrace(sExcept, trace, throwaway)
  write (6, *) trace
end subroutine reportexc

subroutine testnone(fib, test)
  use sidl
  use ExceptionTest_Fib
  use sidl_BaseInterface
  use sidl_SIDLException
  implicit none
  type(ExceptionTest_Fib_t)      :: fib
  type(sidl_BaseInterface_t)     :: iExcept
  type(sidl_SIDLException_t)     :: sExcept
  integer (kind=sidl_int) :: test
  integer (kind=sidl_int) :: retval
  type(sidl_BaseInterface_t) :: throwaway

  call starttest(test)
  call getFib(fib, 10, 25, 200, 0, retval, iExcept)
  if (is_null(iExcept)) then
     call reporttest(.true., test)
     write (6, 100) retval
  else
     call reporttest(.false., test)
     call cast(iExcept, sExcept, throwaway)
     call reportexc(sExcept)
     call deleteRef(sExcept, throwaway)
     call deleteRef(iExcept, throwaway)
  endif
100 format ('fib= ', I4)
end subroutine testnone

subroutine testneg(fib, test)
  use sidl
  use sidl_BaseInterface
  use sidl_SIDLException
  use ExceptionTest_Fib
  implicit none
  type(ExceptionTest_Fib_t)      :: fib
  type(sidl_BaseInterface_t)     :: iExcept
  type(sidl_SIDLException_t)     :: sExcept
  integer (kind=sidl_int) :: test
  integer (kind=sidl_int) :: retval
  logical                        :: isone
  type(sidl_BaseInterface_t) :: throwaway

  call starttest(test)
  call getFib(fib, -1, 10, 10, 0, retval, iExcept)
  if (is_null(iExcept)) then
     call reporttest(.false., test)
     write (6, 100) retval
  else
     call isType(iExcept, &
          'ExceptionTest.NegativeValueException', isone, throwaway)
     if (isone .eqv. .true.) then
        call reporttest(.true., test)
     else
        call reporttest(.false., test)
     endif
     call cast(iExcept, sExcept, throwaway)
     call reportexc(sExcept)
     call deleteRef(sExcept, throwaway)
     call deleteRef(iExcept, throwaway)
  endif
100 format ('fib= ', I4)
end subroutine testneg

subroutine testdeep(fib, test)
  use sidl
  use sidl_BaseInterface
  use sidl_SIDLException
  use ExceptionTest_Fib
  implicit none
  type(ExceptionTest_Fib_t)      :: fib
  type(sidl_BaseInterface_t)     :: iExcept
  type(sidl_SIDLException_t)     :: sExcept
  integer (kind=sidl_int) :: test
  integer (kind=sidl_int) :: retval
  logical                        :: isone
  type(sidl_BaseInterface_t) :: throwaway

  call starttest(test)
  call getFib (fib, 10, 1, 100, 0, retval, iExcept)
  if (is_null(iExcept)) then
     call reporttest(.false., test)
     write (6, 100) retval
  else
     call isType(iExcept, 'ExceptionTest.TooDeepException', &
          isone, throwaway)
     if (isone .eqv. .true.) then
        call reporttest(.true., test)
     else
        call reporttest(.false., test)
     endif
     call cast(iExcept, sExcept, throwaway)
     call reportexc(sExcept)
     call deleteRef(sExcept, throwaway)
     call deleteRef(iExcept, throwaway)
  endif
100 format ('fib= ', I4)
end subroutine testdeep

subroutine testbig(fib, test)
  use sidl
  use sidl_BaseInterface
  use sidl_SIDLException
  use ExceptionTest_Fib
  implicit none
  type(ExceptionTest_Fib_t)      :: fib
  type(sidl_BaseInterface_t)     :: iExcept
  type(sidl_SIDLException_t)     :: sExcept
  integer (kind=sidl_int) :: test
  integer (kind=sidl_int) :: retval
  integer (kind=sidl_int), parameter :: n = 10, max_depth = 100, &
       depth = 0, maxvalue = 1
  logical                        :: isone
  type(sidl_BaseInterface_t) :: throwaway

  call starttest(test)
  call getFib(fib, n, max_depth, depth, maxvalue, retval, iExcept)
  if (is_null(iExcept)) then
     call reporttest(.false., test)
     write (6, 100) retval
  else
     call isType(iExcept, 'ExceptionTest.TooBigException', &
          isone, throwaway)
     if (isone .eqv. .true.) then
        call reporttest(.true., test)
     else
        call reporttest(.false., test)
     endif
     call cast(iExcept, sExcept, throwaway)
     call reportexc(sExcept)
     call deleteRef(sExcept, throwaway)
     call deleteRef(iExcept, throwaway)
  endif
100 format ('fib= ', I4)
end subroutine testbig


program exceptionclient
  use sidl
  use ExceptionTest_Fib
  use synch_RegOut
  implicit none
  integer (kind=sidl_int)  :: test
  type(synch_RegOut_t)            :: tracker
  type(ExceptionTest_Fib_t)       :: fib
  type(sidl_BaseInterface_t) :: throwaway

  call getInstance(tracker, throwaway)
  call setExpectations(tracker, 4, throwaway)
  call new(fib, throwaway)

  test = 1

  call writeComment(tracker, 'No Exception test            ', throwaway)
  call testnone(fib, test)
  call writeComment(tracker, 'Negative Value Exception test', throwaway)
  call testneg(fib, test)
  call writeComment(tracker, 'Too Deep Exception test      ', throwaway)
  call testdeep(fib, test)
  call writeComment(tracker, 'Too Big Exception test       ', throwaway)
  call testbig(fib, test)

  call deleteRef(fib, throwaway)
  call close(tracker, throwaway)
  call deleteRef(tracker, throwaway)
end program exceptionclient
